# pylint: disable=missing-docstring

import json
from os.path import join
from pathlib import Path

import yaml

from ._tools import SaveLoad, JSEncoder


class Local(SaveLoad):
    '''
    '''

    @staticmethod
    def _check_file(path: str) -> None:
        '''
        Method checks if a file exists and extension is allowed.

        Args:

        path(str):
            The full path to the file.
        '''

        if not Path(path).is_file():
            raise ValueError(f'Provided path {path} is not a file!')

        SaveLoad._get_sufix(path, only_verify=True)

    def save_local(
        self,
        fullname: str,
        encoding: str = 'utf8',
        ensure_ascii: bool = True,
        exclude_none: bool = False,
        exclude_unset: bool = True
    ):
        '''
        '''

        parent_path, file_name, file_ext = self._prepare_path(fullname, make_parent=True)
        file_format = self._get_format(fullname)

        if file_format == 'json':
            with open(join(parent_path, f'{file_name}.{file_ext}'), 'w', encoding=encoding) as dump_file:
                dump_file.write(
                    json.dumps(
                        self.dict(exclude_unset=exclude_unset, exclude_none=exclude_none),
                        ensure_ascii=ensure_ascii,
                        cls=JSEncoder
                    )
                )

        elif file_format == 'yaml':
            with open(join(parent_path, f'{file_name}.{file_ext}'), 'w', encoding=encoding) as dump_file:
                yaml.dump(
                    self.dict(exclude_unset=exclude_unset, exclude_none=exclude_none),
                    dump_file,
                    default_flow_style=False
                )

        elif file_format == 'avro':
            # TODO: Implement a procedure for saving avro file to a Local File.
            raise NotImplementedError('avro doesn\'t support now')

        else:
            raise NotImplementedError('unknown file extension')

    @classmethod
    def load_local(cls, fullname: str, encoding: str = 'utf8') -> SaveLoad:
        '''
        '''

        cls._check_file(fullname)
        file_format = cls._get_format(fullname)

        if file_format == 'json':
            return cls.parse_file(fullname, encoding=encoding)

        elif file_format == 'yaml':
            with open(fullname, 'r', encoding=encoding) as src_file:
                return cls.parse_obj(yaml.safe_load(src_file))

        elif file_format == 'avro':
            # TODO: Implement a procedure for loadding avro file from a Local filesystem.
            raise NotImplementedError('avro doesn\'t support now')

        else:
            raise NotImplementedError('unknown file extension')
