# pylint: disable=missing-docstring

from warnings import warn

from google.cloud import firestore

from ._tools import SaveLoad


class Firestore(SaveLoad):
    '''
    '''

    def save_firestore(
        self,
        path: str,
        owerride: bool = False,
        raise_exists: bool = True,
        exclude_none: bool = False,
        exclude_unset: bool = True
    ) -> str | None:
        '''
        '''

        fs_client = firestore.Client()

        if len(path.split('/')) % 2:

            # path contains collection only
            return fs_client.collection(path).add(self.dict(
                exclude_unset=exclude_unset, exclude_none=exclude_none
            ))[1].id

        elif owerride or not fs_client.document(path).get().exists:

            # path contains document and override allowed or document does\'t exists
            fs_client.document(path).set(
                self.dict(
                    exclude_unset=exclude_unset, exclude_none=exclude_none
                )
            )

        else:

            msg = f'The document {path} exists and override don\'t allowed!'
            if raise_exists:
                raise RuntimeError(msg)
            else:
                warn(msg)

    @classmethod
    def load_firestore(cls, path: str) -> SaveLoad | None:
        '''
        '''

        fs_client = firestore.Client()

        if not fs_client.document(path).get().exists:
            raise ValueError(f'The document {path} does not exist!')

        data = fs_client.document(path).get().to_dict()
        cls._validate_schemas(data)

        return cls.parse_obj(data)
