# pylint: disable=missing-docstring

import json

import yaml
from google.cloud import storage as gcs

from ._tools import SaveLoad, JSEncoder


class GCS(SaveLoad):
    '''
    '''

    def save_gcs(
        self,
        fullname: str,
        storage_class: str = 'STANDARD',
        encoding: str = 'utf8',
        ensure_ascii: bool = True,
        exclude_none: bool = False,
        exclude_unset: bool = True
    ):
        '''
        '''

        bucket_name, blob_name = self._prepare_gcs_uri(fullname)
        file_format = self._get_format(fullname)

        bucket = gcs.Client().get_bucket(bucket_name)

        if file_format == 'json':

            data = json.dumps(
                self.dict(exclude_unset=exclude_unset, exclude_none=exclude_none),
                ensure_ascii=ensure_ascii,
                cls=JSEncoder
            )
            content_type = 'application/json'

        elif file_format == 'yaml':
            data = yaml.dump(
                self.dict(exclude_unset=exclude_unset, exclude_none=exclude_none),
                default_flow_style=False
            )
            content_type = 'application/yaml'

        elif file_format == 'avro':
            # TODO: Implement a procedure for saving avro file to Google Cloud Storage.
            raise NotImplementedError('avro doesn\'t support now')

        else:
            raise NotImplementedError('unknown file extension')

        bucket.blob(blob_name).upload_from_string(data, content_type=content_type)

    @classmethod
    def load_gcs(cls, fullname: str, encoding: str = 'utf8') -> SaveLoad:
        '''
        '''

        bucket_name, blob_name = cls._prepare_gcs_uri(fullname)
        file_format = cls._get_format(fullname)

        bucket = gcs.Client().get_bucket(bucket_name)
        blob = bucket.blob(blob_name)

        if file_format == 'json':
            data = json.loads(blob.download_as_text(encoding=encoding))

        elif file_format == 'yaml':
            data = yaml.safe_load(blob.download_as_text(encoding=encoding))

        elif file_format == 'avro':
            # TODO: Implement a procedure for loadding avro file from Google Cloud Storage.
            raise NotImplementedError('avro doesn\'t support now')

        else:
            raise NotImplementedError('unknown file extension')

        cls._validate_schemas(data)

        return cls.parse_obj(data)
